import * as helpers from "./modules/helpers.js";

const navigation = {
  home: {
    label: "Home",
    path: "/",
    command: "home",
    depth: 0,
    submenu: {
      menu: {
        label: "Menu",
        path: "/menu",
        command: "menu",
        depth: 1,
        submenu: {
          call: {
            label: "Call",
            path: "/menu/call",
            command: "call",
            depth: 2,
            submenu: {
              emergency: {
                label: "Emergency",
                command: "emergency",
                path: "/menu/call/emergency",
                depth: 3,
                message: "Calling Emergency",
              },
              contact1: {
                label: "Flute Inc.",
                command: "contact A",
                path: "/menu/call/contact1",
                depth: 3,
                message: "Calling Flute Inc.",
              },
              contact2: {
                label: "Brays AG",
                command: "contact B",
                path: "/menu/call/contact2",
                depth: 3,
                message: "Calling Brays AG",
              },
            },
          },
          check: {
            label: "Check",
            path: "/menu/check",
            command: "check",
            depth: 2,
            submenu: {
              daily: {
                label: "Daily",
                command: "daily",
                path: "/menu/check/daily",
                depth: 3,
                message: "Displaying Daily Checklist",
              },
              weekly: {
                label: "Weekly",
                command: "weekly",
                path: "/menu/check/weekly",
                depth: 3,
                message: "Displaying Weekly Checklist",
              },
              monthly: {
                label: "Monthly",
                command: "monthly",
                path: "/menu/check/monthly",
                depth: 3,
                message: "Displaying Monthly Checklist",
              },
            },
          },
          apps: {
            label: "Apps",
            path: "/menu/apps",
            command: "apps",
            depth: 2,
            submenu: {
              record: {
                label: "Record",
                command: "record",
                path: "/menu/apps/record",
                depth: 3,
                message: "Opening Recording App",
              },
              breakout: {
                label: "Breakout",
                command: "breakout",
                path: "/menu/apps/breakout",
                depth: 3,
                message: "Opening Breakout App",
              },
              statistics: {
                label: "Statistics",
                command: "statistics",
                path: "/menu/apps/statistics",
                depth: 3,
                message: "Opening Statistics App",
              },
            },
          },
        },
      },
    },
  },
};

// init application
function init() {
  // build navigation
  const appEl = document.getElementById("app");
  const navigationEl = document.createElement("nav");
  navigationEl.id = "navigation";
  const listEl = document.createElement("ul");
  navigationEl.appendChild(listEl);
  appEl.appendChild(navigationEl);

  // navigate to home
  navigateTo("/");

  // add two button controls for menu
  document.addEventListener("keydown", onKeyDown);
}

// navigate to page
function navigateTo(path) {
  let navObj = helpers.findNestedObject(navigation, "path", path);
  updateNavigation(navObj);
}

// update Navigation
function updateNavigation(object) {
  // update navigation
  const listEl = document.createElement("ul");

  // add navigation items
  for (let key in object.submenu) {
    let itemEl = buildNavigationItemEl(object.submenu[key]);
    listEl.appendChild(itemEl);
  }

  // add "special back" button
  if (object.depth > 0) {
    let backItemEl;
    if (object.depth === 1)
      backItemEl = buildNavigationItemEl({
        label: "Close",
        path: "/",
        command: "close",
      });
    else if (object.depth === 2)
      backItemEl = buildNavigationItemEl({
        label: "Back",
        path: "/menu",
        command: "back",
      });
    else if (object.depth === 3)
      backItemEl = buildNavigationItemEl({
        label: "Exit",
        path: "/",
        command: "exit",
      });

    listEl.appendChild(backItemEl);
  }

  const navigationEl = document.getElementById("navigation");
  navigationEl.replaceChildren(listEl);

  selectNavigationItem(1);

  // build navigation item
  function buildNavigationItemEl(item) {
    const itemEl = document.createElement("li");
    itemEl.className = "link";
    const linkEl = document.createElement("a");
    linkEl.href = item.path;
    linkEl.innerHTML = item.label;
    linkEl.dataset.command = item.command;
    itemEl.appendChild(linkEl);

    if (item.label.toLowerCase() !== item.command.toLowerCase()) {
      const commandEl = document.createElement("div");
      commandEl.className = "command";
      commandEl.innerHTML = item.command;
      itemEl.appendChild(commandEl);
    }

    linkEl.addEventListener("click", onMouseClick);

    return itemEl;
  }
}

// ==================
// Mouse Navigation Handling

// on menu item mouseclick
function onMouseClick(event) {
  event.preventDefault();
  const path = event.target.getAttribute("href");
  navigateTo(path);
}

// ===================
// Keyboard Navigation Handling

// on keydown event
function onKeyDown(event) {
  event.preventDefault();
  if (event.code === "Tab" || event.code === "ArrowDown")
    selectNextNavigationItem();
  else if (event.code === "ArrowUp") selectPreviousNavigationItem();
  else if (event.code === "Enter" || event.code === "ArrowRight") {
    const currentItem = document.querySelector(
      "#navigation > ul > li[data-selected] a"
    );
    const path = currentItem.getAttribute("href");
    navigateTo(path);
  }
}

// select specific navigation item
function selectNavigationItem(n) {
  // unselect all items
  const navigationList = document.querySelectorAll("#navigation li");
  for (let item of navigationList) delete item.dataset.selected;

  // select requested element
  const navigationItem = document.querySelector(
    `#navigation > ul > li:nth-child(${n})`
  );
  navigationItem.dataset.selected = "";
}

// select next navigation item
function selectNextNavigationItem() {
  // get currently selected element
  const currentItem = document.querySelector(
    "#navigation > ul > li[data-selected]"
  );
  if (currentItem.nextSibling) {
    delete currentItem.dataset.selected;
    currentItem.nextSibling.dataset.selected = "";
  } else selectNavigationItem(1);
}

// select previous navigation item
function selectPreviousNavigationItem() {
  // get currently selected element
  let currentItem = document.querySelector(
    "#navigation > ul > li[data-selected]"
  );
  if (currentItem.previousSibling) {
    delete currentItem.dataset.selected;
    currentItem.previousSibling.dataset.selected = "";
  } else {
    let childCount = document.querySelector("#navigation > ul").children.length;
    selectNavigationItem(childCount);
  }
}

init();
